<?php
namespace Ep\Hak\Service;

use Ep\Hak\HakService;
use Ep\WordPress\Service\AbstractWordPressService;
use WC_Cart;
use Zend\Form\FormInterface;

/**
 * Class CartService
 *
 * @package Ep\Hak\Service
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-hak
 */
class CartService extends AbstractWordPressService
{
	/** @var WC_Cart */
	protected $cart;

	public function attach(): void
	{
		$config = $this->getConfig('hak');

		if ('1' === $config['enabled']) {
			add_action('ep/hak/connect', [$this, 'applyDiscount']);
			add_action('ep/hak/disconnect', [$this, 'removeDiscount']);

			add_action('woocommerce_add_to_cart', [$this, 'onCartChanged']);
			add_action('woocommerce_update_cart_action_cart_updated', [$this, 'onCartChanged']);

			add_action('ep/cart/after_cart', [$this, 'renderForm']);
			add_action('init', [$this, 'handleForm'], 20);

			if (is_object(WC()->cart)) {
				WC()->cart->get_cart();
			}
		}
	}

	/**
	 * Vraća trenutnu košaricu.
	 *
	 * @return WC_Cart
	 */
	protected function getCart()
	{
		if (null === $this->cart) {
			$cart = WC()->cart;

			if ($cart instanceof WC_Cart) {
				$cart->get_cart();
				$this->cart = $cart;
			}
		}

		return $this->cart;
	}

	/**
	 * Poziva se kada god dođe do promjene u košarici.
	 */
	public function onCartChanged()
	{
		/** @var HakService $hak */
		$hak = $this->services->get('Hak');

		if ($hak->validate()) {
			$this->applyDiscount();
		} else {
			$this->removeDiscount();
		}
	}

    /**
     * Vraća true ako je HAK popust primjenjen na košaricu, false inače.
     *
     * @param string $code
     *
     * @return bool
     */
	public function hasDiscount(string $code = HakService::COUPON_CODE): bool
	{
	    foreach ($this->getCart()->get_applied_coupons() as $coupon) {
			if ($coupon === $code) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Primjenjuje HAK popust na košaricu.
	 */
	public function applyDiscount()
	{
        $cart = $this->getCart();

		if (!$this->hasDiscount(HakService::COUPON_CODE)) {
			$cart->apply_coupon(HakService::COUPON_CODE);
		}

        if (!$this->hasDiscount(HakService::DYNAMIC_COUPON_CODE)) {
            $dynamicCoupon = new \WC_Coupon(HakService::DYNAMIC_COUPON_CODE);

            if (apply_filters('woocommerce_coupon_is_valid', false, $dynamicCoupon)) {
                $cart->apply_coupon(HakService::DYNAMIC_COUPON_CODE);

                if (($index = array_search(HakService::DYNAMIC_COUPON_CODE, $cart->applied_coupons)) !== -1) {
                    unset($cart->applied_coupons[$index]);
                    array_unshift($cart->applied_coupons, HakService::DYNAMIC_COUPON_CODE);
                }
            }
        }
	}

    /**
     * Uklanja HAK kupon iz košarice kada se korisnik odspoji od HAK-a.
     *
     * @param string $code
     */
	public function removeDiscount(): void
	{
	    $showNotice = false;

	    foreach ([HakService::COUPON_CODE, HakService::DYNAMIC_COUPON_CODE] as $code) {
            if ($this->hasDiscount($code)) {
                $this->getCart()->remove_coupon($code);
                $showNotice = true;
            }
        }

        if ($showNotice) {
            wc_add_notice('HAK popust uklonjen.', 'error');
        }
	}

	public function handleForm()
	{
		if ('POST' === $_SERVER['REQUEST_METHOD'] && isset($_POST['hakMemberId'])) {
			/** @var FormInterface $form */
			$form = $this->services->get('hak-membership');
			$form->setData($_POST);

			if ($form->isValid()) {
				/** @var HakService $hak */
				$hak = $this->services->get('Hak');
				$data = $form->getData();
				$data['memberId'] = $data['hakMemberId'];

				$hak->setMember($data);

				if ($hak->validate()) {
					$hak->connect();
					wc_add_notice('Uspješno ste povezali vaš račun s HAK-om!', 'success');
				} else {
					wc_add_notice('Nevažeći broj članske iskaznice.', 'error');
				}
			} else {
				$this->removeDiscount();
			}
		}
	}

	/**
	 * Renderira formu ispod košarice.
	 */
	public function renderForm()
	{
		$config = $this->getConfig('hak');
		/** @var HakService $hak */
		$hak = $this->services->get('Hak');

		if ($hak->validate()) {
			return;
		}

		/** @var FormInterface $form */
		$form = $this->services->get('hak-membership');
		$form->setAttribute('class', 'ep-hak-form');
		$form->remove('disconnect');

		echo $this->getView()->render('hak/cart-form.twig', [
			'form' => $form,
			'discount' => $config['discountPercentage'],
		]);
	}
}